@@ -1,14 +1,16 @@
 #! /usr/bin/env python
 # -*- coding: utf-8 -*_
 # Author: Yunlong Feng <ylfeng@ir.hit.edu.cn>
+
 import os
-import torch
 import itertools
 import regex as re
-import transformers
-from typing import Union, List
-
 from packaging import version
+from typing import Union, List
+from argparse import ArgumentParser
+
+import torch
+import transformers
 from transformers import AutoTokenizer, AutoConfig
 from transformers import cached_path, TensorType, BatchEncoding
 from transformers.file_utils import is_remote_url
@@ -18,6 +20,7 @@ transformers_version = version.parse(transformers.__version__)
 from ltp.algorithms import Trie, eisner, split_sentence
 from ltp.transformer_multitask import TransformerMultiTask as Model
 from ltp.utils import length_to_mask, get_entities, fake_import_pytorch_lightning
+from ltp.patchs import model_patch_4_1_3
 
 try:
     from torch.hub import _get_torch_home
@@ -33,12 +36,18 @@ LTP_CACHE = os.getenv("LTP_CACHE", default_cache_path)
 
 model_map = {
     'base': 'http://39.96.43.154/ltp/v3/base.tgz',
+    'base1': 'http://39.96.43.154/ltp/v3/base1.tgz',
+    'base2': 'http://39.96.43.154/ltp/v3/base2.tgz',
     'small': 'http://39.96.43.154/ltp/v3/small.tgz',
-    'tiny': 'http://39.96.43.154/ltp/v3/tiny.tgz'
+    'tiny': 'http://39.96.43.154/ltp/v3/tiny.tgz',
+    'GSD': 'http://39.96.43.154/ltp/ud/gsd.tgz',
+    'GSD+CRF': 'http://39.96.43.154/ltp/ud/gsd_crf.tgz',
+    'GSDSimp': 'http://39.96.43.154/ltp/ud/gsd.tgz',
+    'GSDSimp+CRF': 'http://39.96.43.154/ltp/ud/gsd_crf.tgz',
 }
 
 
-def no_gard(func):
+def no_grad(func):
     def wrapper(*args, **kwargs):
         with torch.no_grad():
             return func(*args, **kwargs)
@@ -124,13 +133,23 @@ class LTP(object):
             fake_import_pytorch_lightning()
             ckpt = torch.load(os.path.join(path, "ltp.model"), map_location=self.device)
 
+        model_patch_4_1_3(ckpt)
+
         self.cache_dir = path
-        config = AutoConfig.for_model(**ckpt['transformer_config'])
-        self.model = Model(ckpt['model_config'], config=config).to(self.device)
+        transformer_config = ckpt['transformer_config']
+        transformer_config['torchscript'] = True
+        config = AutoConfig.for_model(**transformer_config)
+
+        parser = ArgumentParser()
+        parser = Model.add_model_specific_args(parser)
+        model_args = parser.parse_args(args=[], namespace=ckpt['model_config'])
+
+        self.model = Model(model_args, config=config).to(self.device)
         self.model.load_state_dict(ckpt['model'], strict=False)
         self.model.eval()
-        self.max_length = self.model.transformer.config.max_position_embeddings
+
         self.seg_vocab = ckpt.get('seg', [WORD_MIDDLE, WORD_START])
+        self.seg_vocab_dict = {tag: idx for idx, tag in enumerate(self.seg_vocab)}
         self.pos_vocab = ckpt.get('pos', [])
         self.ner_vocab = ckpt.get('ner', [])
         self.dep_vocab = ckpt.get('dep', [])
@@ -138,22 +157,35 @@ class LTP(object):
         self.srl_vocab = [re.sub(r'ARG(\d)', r'A\1', tag.lstrip('ARGM-')) for tag in ckpt.get('srl', [])]
         self.tokenizer = AutoTokenizer.from_pretrained(path, config=self.model.transformer.config, use_fast=True)
         self.trie = Trie()
+        self._model_version = ckpt.get('version', None)
 
     def __str__(self):
-        return f"LTP {self.version} on {self.device}"
+        return f"LTP {self.version} on {self.device} (model version: {self.model_version}) "
 
     def __repr__(self):
-        return f"LTP {self.version} on {self.device}"
+        return f"LTP {self.version} on {self.device} (model version: {self.model_version}) "
+
+    @property
+    def available_models(self):
+        return model_map.keys()
 
     @property
     def version(self):
         from ltp import __version__ as version
         return version
 
+    @property
+    def model_version(self):
+        return self._model_version or 'unknown'
+
+    @property
+    def max_length(self):
+        return self.model.transformer.config.max_position_embeddings
+
     def init_dict(self, path, max_window=None):
         self.trie.init(path, max_window)
 
-    def add_words(self, words, max_window=4):
+    def add_words(self, words, max_window=None):
         self.trie.add_words(words)
         self.trie.max_window = max_window
 
@@ -172,7 +204,7 @@ class LTP(object):
             matching.append(matching_pos)
         return matching
 
-    @no_gard
+    @no_grad
     def _seg(self, tokenizerd, is_preseged=False):
         input_ids = tokenizerd['input_ids'].to(self.device)
         attention_mask = tokenizerd['attention_mask'].to(self.device)
@@ -192,10 +224,11 @@ class LTP(object):
         if is_preseged:
             segment_output = None
         else:
-            segment_output = torch.argmax(self.model.seg_classifier(char_input)[0], dim=-1).cpu().numpy()
+            segment_output = self.model.seg_classifier.forward(char_input, is_processed=True)
+            segment_output = segment_output.decoded or torch.argmax(segment_output.logits, dim=-1).cpu().numpy()
         return word_cls, char_input, segment_output, length
 
-    @no_gard
+    @no_grad
     def seg(self, inputs: Union[List[str], List[List[str]]], truncation: bool = True, is_preseged=False):
         """
         分词
@@ -232,10 +265,10 @@ class LTP(object):
             matches = self.seg_with_dict(inputs, tokenized, batch_prefix)
             for sent_match, sent_seg in zip(matches, seg):
                 for start, end in sent_match:
-                    sent_seg[start] = 0
-                    sent_seg[start + 1:end] = 1
+                    sent_seg[start] = self.seg_vocab_dict[WORD_START]
+                    sent_seg[start + 1:end] = self.seg_vocab_dict[WORD_MIDDLE]
                     if end < len(sent_seg):
-                        sent_seg[end] = 0
+                        sent_seg[end] = self.seg_vocab_dict[WORD_START]
 
         if is_preseged:
             sentences = inputs
@@ -257,7 +290,11 @@ class LTP(object):
             for source_text, length, encoding, seg_tag, preffix in \
                     zip(inputs, lengths, tokenized.encodings, segment_output, batch_prefix):
                 offsets = encoding.offsets[1:length + 1]
-                text = [source_text[start:end] for start, end in offsets]
+                text = []
+                last_offset = None
+                for start, end in offsets:
+                    text.append('' if last_offset == (start, end) else source_text[start:end])
+                    last_offset = (start, end)
 
                 for idx in range(1, length):
                     current_beg = offsets[idx][0]
@@ -277,15 +314,19 @@ class LTP(object):
 
         word_input = torch.gather(hidden, dim=1, index=word_idx)  # 每个word第一个char的向量
 
-        word_cls_input = torch.cat([cls, word_input], dim=1)
-        word_cls_mask = length_to_mask(torch.as_tensor(word_length, device=self.device) + 1)
-        word_cls_mask[:, 0] = False
+        if len(self.dep_vocab) + len(self.sdp_vocab) > 0:
+            word_cls_input = torch.cat([cls, word_input], dim=1)
+            word_cls_mask = length_to_mask(torch.as_tensor(word_length, device=self.device) + 1)
+            word_cls_mask[:, 0] = False
+        else:
+            word_cls_input, word_cls_mask = None, None
+
         return sentences, {
             'word_cls': cls, 'word_input': word_input, 'word_length': word_length,
             'word_cls_input': word_cls_input, 'word_cls_mask': word_cls_mask
         }
 
-    @no_gard
+    @no_grad
     def pos(self, hidden: dict):
         """
         词性标注
@@ -295,29 +336,34 @@ class LTP(object):
         Returns:
             pos: 词性标注结果
         """
-        postagger_output = self.model.pos_classifier(hidden['word_input'])[0]
-        postagger_output = torch.argmax(postagger_output, dim=-1).cpu().numpy()
+        if len(self.pos_vocab) == 0:
+            return []
+        postagger_output = self.model.pos_classifier(hidden['word_input'], is_processed=True)
+        postagger_output = postagger_output.decoded or torch.argmax(postagger_output.logits, dim=-1).cpu().numpy()
         postagger_output = convert_idx_to_name(postagger_output, hidden['word_length'], self.pos_vocab)
         return postagger_output
 
-    @no_gard
-    def ner(self, hidden: dict):
+    @no_grad
+    def ner(self, hidden: dict, as_entities=True):
         """
         命名实体识别
         Args:
             hidden: 分词时所得到的中间表示
+            as_entities: 是否以 Entity(Type, Start, End) 的形式返回
 
         Returns:
             pos: 命名实体识别结果
         """
+        if len(self.ner_vocab) == 0:
+            return []
         ner_output = self.model.ner_classifier.forward(
-            hidden['word_input'], word_attention_mask=hidden['word_cls_mask'][:, 1:]
-        )[0]
-        ner_output = torch.argmax(ner_output, dim=-1).cpu().numpy()
+            hidden['word_input'], word_attention_mask=hidden['word_cls_mask'][:, 1:], is_processed=True
+        )
+        ner_output = ner_output.decoded or torch.argmax(ner_output.logits, dim=-1).cpu().numpy()
         ner_output = convert_idx_to_name(ner_output, hidden['word_length'], self.ner_vocab)
-        return [get_entities(ner) for ner in ner_output]
+        return [get_entities(ner) for ner in ner_output] if as_entities else ner_output
 
-    @no_gard
+    @no_grad
     def srl(self, hidden: dict, keep_empty=True):
         """
         语义角色标注
@@ -327,10 +373,13 @@ class LTP(object):
         Returns:
             pos: 语义角色标注结果
         """
-        srl_score, srl_output, _ = self.model.srl_classifier.forward(
+        if len(self.srl_vocab) == 0:
+            return []
+        srl_output = self.model.srl_classifier.forward(
             input=hidden['word_input'],
-            word_attention_mask=hidden['word_cls_mask'][:, 1:]
-        )[0]
+            word_attention_mask=hidden['word_cls_mask'][:, 1:],
+            is_processed=True
+        ).decoded
         srl_entities = get_entities_with_list(srl_output, self.srl_vocab)
 
         srl_labels_res = []
@@ -346,24 +395,29 @@ class LTP(object):
             ]
         return srl_labels_res
 
-    @no_gard
-    def dep(self, hidden: dict, fast=True):
+    @no_grad
+    def dep(self, hidden: dict, fast=True, as_tuple=True):
         """
         依存句法树
         Args:
             hidden: 分词时所得到的中间表示
             fast: 启用 fast 模式时，减少对结果的约束，速度更快，相应的精度会降低
+            as_tuple: 返回的结果是否为 (idx, head, rel) 的格式，否则返回 heads, rels
 
         Returns:
             依存句法树结果
         """
+        if len(self.dep_vocab) == 0:
+            return []
         word_attention_mask = hidden['word_cls_mask']
-        dep_arc, dep_label = self.model.dep_classifier.forward(
+        result = self.model.dep_classifier.forward(
             input=hidden['word_cls_input'],
-            word_attention_mask=word_attention_mask[:, 1:]
-        )[0]
+            word_attention_mask=word_attention_mask[:, 1:],
+            is_processed=True
+        )
+        dep_arc, dep_label = result.arc_logits, result.rel_logits
         dep_arc[:, 0, 1:] = float('-inf')
-        dep_arc.diagonal(0, 1, 2)[1:].fill_(float('-inf'))
+        dep_arc.diagonal(0, 1, 2).fill_(float('-inf'))
         dep_arc = dep_arc.argmax(dim=-1) if fast else eisner(dep_arc, word_attention_mask)
 
         dep_label = torch.argmax(dep_label, dim=-1)
@@ -372,7 +426,7 @@ class LTP(object):
         dep_arc[~word_attention_mask] = -1
         dep_label[~word_attention_mask] = -1
 
-        arc_pred = [
+        head_pred = [
             [item for item in arcs if item != -1]
             for arcs in dep_arc[:, 1:].cpu().numpy().tolist()
         ]
@@ -380,40 +434,51 @@ class LTP(object):
             [self.dep_vocab[item] for item in rels if item != -1]
             for rels in dep_label[:, 1:].cpu().numpy().tolist()
         ]
-
+        if not as_tuple:
+            return head_pred, rel_pred
         return [
-            [(idx + 1, arc, rel) for idx, (arc, rel) in enumerate(zip(arcs, rels))]
-            for arcs, rels in zip(arc_pred, rel_pred)
+            [(idx + 1, head, rel) for idx, (head, rel) in enumerate(zip(heads, rels))]
+            for heads, rels in zip(head_pred, rel_pred)
         ]
 
-    @no_gard
-    def sdp(self, hidden: dict, graph=True):
+    @no_grad
+    def sdp(self, hidden: dict, mode: str = 'graph'):
         """
         语义依存图（树）
         Args:
             hidden: 分词时所得到的中间表示
-            graph: 选择是语义依存图还是语义依存树结果
+            mode: ['tree', 'graph', 'mix']
 
         Returns:
             语义依存图（树）结果
         """
+        if len(self.sdp_vocab) == 0:
+            return []
+
         word_attention_mask = hidden['word_cls_mask']
-        sdp_arc, sdp_label = self.model.sdp_classifier(
+        result = self.model.sdp_classifier(
             input=hidden['word_cls_input'],
-            word_attention_mask=word_attention_mask[:, 1:]
-        )[0]
+            word_attention_mask=word_attention_mask[:, 1:],
+            is_processed=True
+        )
+        sdp_arc, sdp_label = result.arc_logits, result.rel_logits
         sdp_arc[:, 0, 1:] = float('-inf')
-        sdp_arc.diagonal(0, 1, 2)[1:].fill_(float('-inf'))  # 避免自指
+        sdp_arc.diagonal(0, 1, 2).fill_(float('-inf'))  # 避免自指
         sdp_label = torch.argmax(sdp_label, dim=-1)
 
-        if graph:
-            # 语义依存图
-            sdp_arc = torch.sigmoid_(sdp_arc) > 0.5
-        else:
+        if mode == 'tree':
             # 语义依存树
             sdp_arc_idx = eisner(sdp_arc, word_attention_mask).unsqueeze_(-1).expand_as(sdp_arc)
-            sdp_arc = torch.zeros_like(sdp_arc, dtype=torch.bool).scatter_(-1, sdp_arc_idx, True)
-        sdp_arc[~word_attention_mask] = False
-        sdp_label = get_graph_entities(sdp_arc, sdp_label, self.sdp_vocab)
+            sdp_arc_res = torch.zeros_like(sdp_arc, dtype=torch.bool).scatter_(-1, sdp_arc_idx, True)
+        elif mode == 'mix':
+            # 混合解码
+            sdp_arc_idx = eisner(sdp_arc, word_attention_mask).unsqueeze_(-1).expand_as(sdp_arc)
+            sdp_arc_res = (sdp_arc.sigmoid_() > 0.5).scatter_(-1, sdp_arc_idx, True)
+        else:
+            # 语义依存图
+            sdp_arc_res = torch.sigmoid_(sdp_arc) > 0.5
+
+        sdp_arc_res[~word_attention_mask] = False
+        sdp_label = get_graph_entities(sdp_arc_res, sdp_label, self.sdp_vocab)
 
         return sdp_label
