@@ -1,7 +1,9 @@
 #! /usr/bin/env python
 # -*- coding: utf-8 -*_
 # Author: Yunlong Feng <ylfeng@ir.hit.edu.cn>
-import torch, math
+
+import math
+import torch
 from torch import nn, Tensor
 from torch.nn import functional as F
 
