@@ -1,8 +1,10 @@
 #! /usr/bin/env python
 # -*- coding: utf-8 -*_
 # Author: Yunlong Feng <ylfeng@ir.hit.edu.cn>
-from typing import Optional
-import torch, math
+# ref: https://github.com/fastnlp/TENER
+
+import math
+import torch
 from torch import Tensor, nn
 import torch.nn.functional as F
 from ltp.nn.ffnn import FFNN
@@ -125,7 +127,7 @@ class RelativeMultiHeadAttn(nn.Module):
         zero_pad = BD.new_zeros(bsz, n_head, max_len, 1)
         BD = torch.cat([BD, zero_pad], dim=-1).view(bsz, n_head, -1, max_len)  # bsz x n_head x (2max_len+1) x max_len
         BD = BD[:, :, :-1, :].view(bsz, n_head, max_len, -1)  # bsz x n_head x 2max_len x max_len
-        _, BD = torch.chunk(BD, dim=-1, chunks=2)
+        BD = BD[:, :, :, :max_len]
         return BD
 
 
